%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Mobile Arts AB
%% Portions created by Mobile Arts are Copyright 2002, Mobile Arts AB
%% All Rights Reserved.''
%% 
%%
-module(http_request).

-include("http.hrl").

-export([send/2]).

send(#request{method = Method,id = Id,
	      scheme = Scheme, address = {Host,Port}, path = Path,
	      pquery = Query, headers = Headers, content = Content,
	      settings = Settings},
     Socket) -> 

    PostData = post_data(Method, Headers, Content),
	
    Message = method(Method) ++ " " ++ Path ++ Query ++ " HTTP/1.1" ++ ?CRLF ++
	headers(Headers) ++
	http_cookie:header(Host, Port, Path, Query, 
			   Settings#client_settings.cookie) ++
	PostData,
    
    http_lib:send(Scheme, Socket, Message).


post_data(Method, Headers, {ContentType, Body}) 
  when Method == post; Method == put ->
    
    content_type_header(ContentType) ++
	content_length_header(length(Body)) ++ 
	?CRLF ++
	case Headers#http_request_h.expect of
	    "100-continue" ->
		"";
	    _ ->
		Body
	end;

post_data(_, _, _) ->
    ?CRLF.


%% Transform from #http_response_h record to HTTP syntax 
%% Note: - Headers have been validated in http.erl, thus we know
%% that the Host and TE field exists.

headers(#http_request_h{expect = Expect,
			host = Host,
			te = TE,
			other = Other}) ->
    HostH = ["Host: ", Host,  ?CRLF],
    ExpectH = case Expect of
		  undefined ->[];
		  _ -> ["Expect: ",Expect, ?CRLF]
	      end,
    TEH = ["TE: ",TE, ?CRLF],
    [HostH, ExpectH, TEH | headers_other(Other)].

headers_other([]) ->
    [];
headers_other([{Key,Value} | Rest]) when atom(Key) ->
    Head = atom_to_list(Key) ++ ": " ++ Value ++ ?CRLF,
    Head ++ headers_other(Rest);

headers_other([{Key,Value} | Rest]) ->
    Head = Key ++ ": " ++ Value ++ ?CRLF,
    Head ++ headers_other(Rest).

content_type_header(ContentType) ->
    "Content-Type: " ++ ContentType ++  ?CRLF.
content_length_header(ContentLength) ->
    "Content-Length: "++integer_to_list(ContentLength) ++ ?CRLF.
    
method(Method) ->
    httpd_util:to_upper(atom_to_list(Method)).





