%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id: snmpa.erl,v 1.1.1.1 2004/07/05 09:16:55 tobiasl Exp $
%%
-module(snmpa).


%%----------------------------------------------------------------------
%% This module contains the user interface to the snmp agent toolkit.
%%----------------------------------------------------------------------

-export([verbosity/2, 
	 
	 current_request_id/0, current_community/0, current_address/0,
	 current_context/0, current_net_if_data/0, 
	 
	 get_symbolic_store_db/0,
	 name_to_oid/1, name_to_oid/2, 
	 oid_to_name/1, oid_to_name/2,
	 int_to_enum/2, int_to_enum/3, 
	 enum_to_int/2, enum_to_int/3,

	 get/2, get/3, get_next/2, get_next/3,
	 info/1, 
	 load_mibs/2, unload_mibs/2, 
	 dump_mibs/0, dump_mibs/1,

	 register_subagent/3, unregister_subagent/2, 

	 send_notification/3, send_notification/4, send_notification/5,
	 send_notification/6,
	 send_trap/3, send_trap/4,

         register_notification_filter/3,
         register_notification_filter/4,
         register_notification_filter/5,
         unregister_notification_filter/1,
         unregister_notification_filter/2,
         which_notification_filter/0,
         which_notification_filter/1,
 
 	 sys_up_time/0, system_start_time/0]).

%% USM functions:
-export([passwd2localized_key/3, localize_key/3]).

%% Agent Capabilities functions
-export([add_agent_caps/2, del_agent_caps/1, get_agent_caps/0]).

%% Audit Trail Log functions
-export([log_to_txt/2, log_to_txt/3, log_to_txt/4, 
	 change_log_size/1]).


%% The old (pre-4.0) agent audit trail log defaults
-define(audit_trail_log_name, "snmpa log"). 
-define(audit_trail_log_file, "snmpa.log").


%% Note that verbosity for the agents is actually only implemented 
%% (properly) for the master agent.
verbosity(all,Verbosity) -> 
    catch snmp_agent:verbosity(sub_agents,Verbosity),
    catch snmp_agent:verbosity(master_agent,Verbosity),
    catch snmp_agent:verbosity(net_if,Verbosity),
    catch snmp_agent:verbosity(mib_server,Verbosity),
    catch snmp_agent:verbosity(note_store,Verbosity),
    catch snmp_symbolic_store:verbosity(Verbosity),
    catch snmp_local_db:verbosity(Verbosity);
verbosity(master_agent,Verbosity) -> 
    catch snmp_agent:verbosity(snmp_master_agent,Verbosity);
verbosity(net_if,Verbosity) -> 
    catch snmp_agent:verbosity(snmp_net_if,Verbosity);
verbosity(note_store,Verbosity) -> 
    catch snmp_agent:verbosity(note_store, Verbosity);
verbosity(mib_server,Verbosity) -> 
    catch snmp_agent:verbosity(mib_server,Verbosity);
verbosity(symbolic_store,Verbosity) -> 
    catch snmp_symbolic_store:verbosity(Verbosity);
verbosity(local_db,Verbosity) -> 
    catch snmp_local_db:verbosity(Verbosity);
verbosity(Agent,{subagents,Verbosity}) -> 
    catch snmp_agent:verbosity(Agent,{sub_agents,Verbosity});
verbosity(Agent,Verbosity) -> 
    catch snmp_agent:verbosity(Agent,Verbosity).


%%-----------------------------------------------------------------
%% Get the symbolic store (internal) database id for faster access to
%% to data.
%% Returns: term()
%%-----------------------------------------------------------------

get_symbolic_store_db() ->
    snmp_symbolic_store:get_db().


%%-----------------------------------------------------------------
%% These 8 functions returns {value, Val} | false
%%-----------------------------------------------------------------
name_to_oid(Name) ->
    snmp_symbolic_store:aliasname_to_oid(Name).

name_to_oid(Db, Name) ->
    snmp_symbolic_store:aliasname_to_oid(Db, Name).

oid_to_name(OID) ->
    snmp_symbolic_store:oid_to_aliasname(OID).

oid_to_name(Db, OID) ->
    snmp_symbolic_store:oid_to_aliasname(Db, OID).

enum_to_int(Name, Enum) ->
    snmp_symbolic_store:enum_to_int(Name, Enum).

enum_to_int(Db, Name, Enum) ->
    snmp_symbolic_store:enum_to_int(Db, Name, Enum).

int_to_enum(Name, Int) ->
    snmp_symbolic_store:int_to_enum(Name, Int).

int_to_enum(Db, Name, Int) ->
    snmp_symbolic_store:int_to_enum(Db, Name, Int).


%%-----------------------------------------------------------------
%% These functions must only be called in the process context
%% where the instrumentation functions are called!
%%-----------------------------------------------------------------
current_request_id()  -> current_get(snmp_request_id).
current_context()     -> current_get(snmp_context).
current_community()   -> current_get(snmp_community).
current_address()     -> current_get(snmp_address).
current_net_if_data() -> current_get(snmp_net_if_data).

current_get(Tag) ->
    case get(Tag) of
	undefined -> false;
	X -> {value, X}
    end.
    

%% -

get(Agent, Vars) -> snmp_agent:get(Agent, Vars).
get(Agent, Vars, Context) -> snmp_agent:get(Agent, Vars, Context).

get_next(Agent, Vars) -> snmp_agent:get_next(Agent, Vars).
get_next(Agent, Vars, Context) -> snmp_agent:get_next(Agent, Vars, Context).


info(Agent) -> snmp_agent:info(Agent).


%% -

dump_mibs()     -> snmp_agent:dump_mibs(snmp_master_agent).
dump_mibs(File) -> snmp_agent:dump_mibs(snmp_master_agent, File).

load_mibs(Agent, Mibs) when list(Mibs) -> 
    snmp_agent:load_mibs(Agent, Mibs).

unload_mibs(Agent, Mibs) when list(Mibs) -> 
    snmp_agent:unload_mibs(Agent, Mibs).


%% -

register_notification_filter(Id, Mod, Data) when atom(Mod) ->
    register_notification_filter(snmp_master_agent, Id, Mod, Data, last).
 
register_notification_filter(Agent, Id, Mod, Data)
  when atom(Agent), atom(Mod) ->
    register_notification_filter(Agent, Id, Mod, Data, last);
register_notification_filter(Agent, Id, Mod, Data)
  when pid(Agent), atom(Mod) ->
    register_notification_filter(Agent, Id, Mod, Data, last);
register_notification_filter(Id, Mod, Data, Where) when atom(Mod) ->
    register_notification_filter(snmp_master_agent, Id, Mod, Data, Where).
 
register_notification_filter(Agent, Id, Mod, Data, Where) ->
    snmp_agent:register_notification_filter(Agent, Id, Mod, Data, Where).
 
unregister_notification_filter(Id) ->
    unregister_notification_filter(snmp_master_agent, Id).
 
unregister_notification_filter(Agent, Id) ->
    snmp_agent:unregister_notification_filter(Agent, Id).
 
which_notification_filter() ->
    which_notification_filter(snmp_master_agent).
 
which_notification_filter(Agent) ->
    snmp_agent:which_notification_filter(Agent).
 
 
%% -

send_notification(Agent, Notification, Recv) ->
    send_notification(Agent, Notification, Recv, "", "", []).

send_notification(Agent, Notification, Recv, Varbinds) ->
    send_notification(Agent, Notification, Recv, "", "", Varbinds).

send_notification(Agent, Notification, Recv, NotifyName, Varbinds) ->
    send_notification(Agent, Notification, Recv, NotifyName, "", Varbinds).

send_notification(Agent, Notification, Recv,NotifyName,ContextName,Varbinds) 
  when list(NotifyName), list(ContextName), list(Varbinds) ->
    snmp_agent:send_trap(Agent, Notification, NotifyName, 
			 ContextName, Recv, Varbinds).

%% Kept for backwards compatibility
send_trap(Agent, Trap, Community) ->
    send_notification(Agent, Trap, no_receiver, Community, "", []).

send_trap(Agent, Trap, Community, Varbinds) ->
    send_notification(Agent, Trap, no_receiver, Community, "", Varbinds).

register_subagent(Agent, SubTree, SubAgent) ->
    snmp_agent:register_subagent(Agent, SubTree, SubAgent).

unregister_subagent(Agent, SubOidOrPid) ->
    snmp_agent:unregister_subagent(Agent, SubOidOrPid).

system_start_time() ->
    [{_, Time}] = ets:lookup(snmp_agent_table, system_start_time),
    Time.

sys_up_time() ->
    % time in 0.01 seconds.
    StartTime = system_start_time(),
    (snmp_misc:now(cs) - StartTime) rem (2 bsl 31).


%%%-----------------------------------------------------------------
%%% USM functions
%%%-----------------------------------------------------------------
passwd2localized_key(Alg, Passwd, EngineID) ->
    snmp_usm:passwd2localized_key(Alg, Passwd, EngineID).

localize_key(Alg, Key, EngineID) ->
    snmp_usm:localize_key(Alg, Key, EngineID).


%%%-----------------------------------------------------------------
%%% Agent Capabilities functions
%%%-----------------------------------------------------------------
add_agent_caps(Oid, Descr) ->
    snmp_standard_mib:add_agent_caps(Oid, Descr).

del_agent_caps(Index) ->
    snmp_standard_mib:del_agent_caps(Index).

get_agent_caps() ->
    snmp_standard_mib:get_agent_caps().


%%%-----------------------------------------------------------------
%%% Audit Trail Log functions (for backward compatibillity)
%%%-----------------------------------------------------------------
log_to_txt(LogDir, Mibs) -> 
    OutFile = "snmpa_log.txt",       
    LogName = ?audit_trail_log_name, 
    LogFile = ?audit_trail_log_file, 
    snmp:log_to_txt(LogName, LogFile, LogDir, Mibs, OutFile).
log_to_txt(LogDir, Mibs, OutFile) -> 
    LogName = ?audit_trail_log_name, 
    LogFile = ?audit_trail_log_file, 
    snmp:log_to_txt(LogName, LogFile, LogDir, Mibs, OutFile).
log_to_txt(LogDir, Mibs, OutFile, LogName) -> 
    LogFile = ?audit_trail_log_file, 
    snmp:log_to_txt(LogName, LogFile, LogDir, Mibs, OutFile).

change_log_size(NewSize) -> 
    LogName = ?audit_trail_log_name, % The old (agent) default
    snmp:change_size(LogName, NewSize).


